<?php
##### rrshared_modsettingsblocks3.php (vv1) A "protected" Include file installed by several of Robin Richmond's mods
/*
The purpose of this Include file is to organize "Mod Settings Blocks" in Admin>>Setup programs.

This file contains PHP functions, a stylesheet, and a Javascript block that contains a
substantial document.ready function and some named functions that are either called from the
document.ready function or from events in HTML elements created by the document.ready function.

For more thorough documentation than this See the TNG Wiki article 
https://tng.lythgoes.net/wiki/index.php?title=Mod_Settings_Blocks
========================================

Mods that define Mod Settings Blocks all
1. Include this file with an include_once statement, and
2. Place their mod settings in an HTML layout table with the class='rrConfigTable'.

These layout tables can be formatted in any fashion, but in all cases so far, they are
formatted just like most of the native TNG settings, with one table row per form field,
where each form field defines a setting.  The essential differences between Mod Setting Blocks
and the way that existing mods typically define settings in Admin>>Setup program is that
Mod Settings Blocks define all of a mod's settings in a bordered, labeled block, so that
the settings associated with a particular mod can be readily identified.

More specifically, 
* With existing mods:
  1. Each new form field (i.e. each new setting) is typically placed in a new table row
     somewhere in (usually at the bottom of) an existing form or subform.
  2. The new options/rows/form fields typically do not say which mod they came from, either
     individually or as a group.
* With Mod Settings Blocks:
  1. Each mod specifies a new layout table, and all of the options for that mod are in
     the mod's layout table.
  2. Each mod's table is wrapped in a bordered div, and that div also displays the mod name
     as a heading.
  3. Placement of the Mod Settings Blocks 
     b. In most cases, they are defined in admin_genconfig.php (Admin>Setup>General Settings),
        and are placed in a new subform called "Robin's Mods", which is placed after the
        last existing subform (Miscellaneous).
     a. Otherwise, they are placed at the bottom of an existing layout table (in a subform, or,
        in Admin>>Stup program that do not have subforms, in the main form layout table).
  4. Within a given form or subform, the mods are sorted by mod name.

This file is distributed with numerous mods, always in the shared mods subfolder RR-shared_mod_includes_v12.0.0.3.
Only one mod will actually install this file, since the %copyfile commands in all of the mods
use the Mod Manager's "protected" flag. 

The number 2 in this filename and in the folder that contains it are version numbers.
If the code in this file must be updated, well, mods that try to install the updated version
will fail to do so if another mod has already installed an older version.  Thus, when this file
must be updated, a new file and folder with a new version number must be created, and all mods
that use this file must be updated so that
- They install the new Include file, and
- The programs thaty they modify Include the new Include file.

In general, when a new version of this file is created, it should not be important that
every installed mod that uses this file be updated to the new mod version. But it is probably prudent to do so

************* Revision History ***************
v3 1 Dec 2019
- (Of course) updated the version number suffixes in the filename and function names.
- Fixed a bug in the code that places a Mod Settings Block in an Admin>>Setup program
  that does not have submenus.
- Added version number suffixes to all Javascript functions, even if they are only called
  internally.
v2 4 Nov 2019
- Added version number suffixes to all PHP functions.
- When a mod settings block is opened, open just that one block. It previously opened
  all block if there were fewer than 4 blocks in the setup program.
- Added a border to the wiki logo hyperlink since the logo didn't look enougth like a button.
- Added the hyperlink cursor to the wiki logo.
- Added PHP functions that define Field Button-like buttons marked with a D to explicitly
  assign default values to options in text and textarea fields. (Version 1 had such buttons
  for checkboxes and selection boxes. These functions are essential redundant with D field
  buttons, but are consistent with the existing default buttons.)
*/
########### Inline code ##########
#### Field Buttons - load library and initialize
if (!isset($rrButtonsAlreadyInstalled)) {
	if ($admtext['rrfb-buttonlabels'] && $rrconfig['FieldButtonFlags'] != '0,0,0,0' ) {
		echo "<script id='rrfblib' type='text/javascript' src='js/fieldbuttonsmod.js'></script>\n";
		$rrButtonsAlreadyInstalled = 1;
?>
	<script type="text/javascript">
		rrButtonsPushFunction ('after', function() {
			//Pass the field button labels & flags to the Field Buttons library.
			rrButtonsSystemSettings ("<?php echo $admtext['rrfb-buttonlabels']; ?>",
				"<?php echo $rrconfig['FieldButtonFlags']; ?>");
		});
	</script>
<?php
	} else { #No field buttons library => we need some dummy functions
		echo "<script>
			function rrButtonsSetVisibility(){};
			function rrButtonsSetVisibilityAll(){};
			function rrButtonsPushFunction() {};
		</script>\n";
	}
}
#### End of Field Buttons setup

################### External PHP Functions ###############
# Generate a default button for a selection box in an Admin>>Setup form
function rrDefaultSelectBox3($fieldID, $defaultvalue) {
	return "<button class='rrSetupDefaultButton'
		onclick=\"$('#$fieldID>option').attr('selected',false); $('#$fieldID>option[value=$defaultvalue]').attr('selected',true); return false;\"
		title=\"Select '$defaultvalue'\">D</button>";
} # End function rrDefaultSelectBox

############################################################
# Generate a Default button for a checkbox in an Admin>>Setup form
function rrDefaultCheckbox3($fieldID, $checked) {
	return "<button class='rrSetupDefaultButton'
		onclick=\"$('#$fieldID').prop('checked',$checked);return false;\"
		title=\"" . ($checked ? 'Check the box' : 'Uncheck the box') . "\">D</button>";
} # End function rrDefaultCheckbox

############################################################
# Generate a Default button for a textarea in an Admin>>Setup form
function rrDefaultTextArea3($fieldID, $valueID) {
	return "<button class='rrSetupDefaultButton' 
		onclick=\"$('#$fieldID').val($('#$valueID').html());return false;\"
		title='Set to default value'>D</button>\n";
} # End function rrDefaultTextArea

############################################################
# Generate a Default button for a text box in an Admin>>Setup form
function rrDefaultText3($fieldID, $defaultvalue) {
$newvalue = addslashes($defaultvalue);
	return "<input type='button' class='rrSetupDefaultButton' 
		onclick=\"$('#$fieldID').val('$newvalue');return false;\"
		title=\" Set to $newvalue\" value='D'>\n";
} # End function rrDefaultText


############################################################
############ document.ready function #######################
# This function can be invoked in (almost) any Admin>>Setup program. 
# It reorganizes the "source" Mod Settings Blocks so that, in they end.
# 1. They are inside an HTML layout table that lays the mod options (i.e. form fields)
#    for an Admin>>Setup program's subform, or, for programs without a subform, the main
#    HTML form layout table.
# 2. They are sorted by mod name.
# 3. They are placed inside a new row in the layout table
#    Note that layout table rows usually contain a single form field, but with Mod Settings
#    Blocks, each row contains a table that contains all of the settings for one mod.
# 4. Each layout table row actually contains a bordered div that contains:
#    a. The mod name as a heading,
#    b. A link to the mod's Wiki article,
#    c. The mod's Mod Settings Block.
?>
<script type='text/javascript'>
/////////////////////////////////////////////
$(function() {
	//Nothing to do if there are no settings tables.
	if (!$('.rrConfigTable').length) return false; 

	var configProgram = window.location.pathname.split("/").pop();
	if (configProgram == 'admin_genconfig.php') {
		//Create a new subform for all mod settings that are defined in Mod Settings Block.
		//Give it the label "Robin's Mods" and id="robin".
		var html = "<tr class='databack tngshadow'><td class='tngshadow>";
		html += "<span class='subhead'><a href='#' onclick='return rrToggleRobin3();' class='togglehead' style='color:black'>";
		html += "<img src='img/tng_expand.gif' title='toggle display' alt='toggle display' width='15' height='15' border='0' id='plusrr'>";
		html += "<strong class='th-indent'>Robin's Mods" +"</strong></a></span><br />";
		html += "<div id='robin' style='display:none'><br><table class='normal'><tbody></tbody></table></div>";
		html += "</td></tr>";
		//The new subform follows the last native subform, 'Miscellaneous'
		$("#misc").closest('tr').after(html);
	}

	//Sort the options tables that Robin's mods have created.
	var rrGroups = Array();
	var rrMods = Array();
	var rrModOptions = Array();
	var rrModSubform = Array();
	var rrCount = 0; //Counts mod that go in Robin's new subforkm
	$('.rrConfigTable').each(function() {
		var modname = $(this).attr('modname');
		rrMods.push(modname);
		rrModOptions[modname] = $(this);
		var subform = $(this).attr('subform');
		if (typeof(subform) == "undefined") {
			if (configProgram == 'admin_genconfig.php') {
				subform = 'robin';
				rrCount++;
			}
			else if (configProgram == 'admin_pedconfig.php')
				subform = 'peddesc';
			else subform = '';
		}
		rrModSubform[modname] = subform;
		$(this).remove();
	});
	rrMods.sort(); //Sort by modname

	var modToOpen = rrGetQueryStringParam3('mod');
	var modDisplayStyle = '';
	//In admin_genconfig only, hide each mod if a modid has been passed in
	if (configProgram == 'admin_genconfig.php' && modToOpen)
		modDisplayStyle = " style='display:none;'";

	//Put each mod into the appropriate subform
	for (rr=0; rr<rrMods.length; rr++) {
		var modname = rrMods[rr];
		var modid = modname.replace(/ /g, '');
		var group = rrModOptions[modname].attr('group');
		if (!group) group = 'Misc'; //Not used for now

		html = "<tr"+modDisplayStyle+" tabindex='-1' id='" +modid+ "' class='rrModOptions'>" +
			"<td colspan='5' class='normal'>" +
			"<div style='float:right;'><input type='submit' name='submit' value='Save All'></div>" +
			"<div class='rrModWrapper'>" +
			"<b style='font-size:110%;'>"+modname+"</b> &nbsp;" +
			"<a href='#' onclick=\"return rrModOptionsOpenWikiPage3('" +modname+ "', 'Mod_Options');\">" +
			"<img src='img/rrshared_wikilogo.gif' border='2' style='height:16px;width:16px;' class='nw rrModAdminWiki'></a>" +
			"<table class='rrModTable normal'>" + rrModOptions[modname].html() + "</table>" +
			"</div></td></tr>";

		//Each mod's content is put a new table row at the end of the form's layout table,
		//or, if there is a subform, in the subform's layout table.
		//(The calls to the .first method is probably be unnecessary, but doesn't hurt.)
		var parentElement = $('form');
		if (rrModSubform[modname])
			parentElement = $('#'+rrModSubform[modname]);
		parentElement.children('table').first().children('tbody').first().append(html);
	}

	//Open the subform section specified by the querystring parameter 'open',
	var subform = rrGetQueryStringParam3('open');
	if (subform) {
		if (subform=='robin') {
			if (modToOpen) {
				$("#"+modToOpen).show(); //Don't toggle the arrow in the heading
				//Open the mod and bring focus to it.
				$("#robin").show();
				$('#'+modToOpen).focus();
			} else { //Open the subform and all mods in it
				rrToggleRobin3();
			}
		} else { //Use the generic function to open the subform
			toggleSection(subform, rrGetImageId3(subform), 'on');
		}
	} else if (modToOpen) {
		//Turn on the display of this mod settings block only. (The other mod settings blocks
		//remain hidden even when the subform is opened.)
		$("#"+modToOpen).show(); 
		//Open the subform without affecting the arrow in its heading,
		//and bring focus to the mod settings block.
		$("#robin").show();
		$('#'+modToOpen).focus();
	} //end if open a subform
}); //end document.ready function

////////////////////////////////////////////////
//Function to toggle the display of the subform for Robin's mods.
//It is different from the generic toggleSection function because each
//mod in the subform must be opened the first time the subform is opened.
function rrToggleRobin3() {
	if ($('#plusrr').attr('src')=='img/tng_expand.gif') {
//	if ($('#robin').css('display')=='block') {
		//Opening the subform. Each mod must be opened, too.
		$('#robin').show();
		$('#plusrr').attr('src', 'img/tng_collapse.gif');
		$('.rrModOptions').show(); //Show each mod
	} else { //Closing the subform 	
		//There is no need to hide each mod
		$('#robin').hide();
		$('#plusrr').attr('src', 'img/tng_expand.gif');
	}
	return false;
} //end function rrToggleRobin3

////////////////////////////////////////////////
//Gets the value of a querystring parameter (duh!)
function rrGetQueryStringParam3(sParam) {
	var sPageURL = window.location.search.substring(1);
	var sURLVariables = sPageURL.split('&');
	for (var i = 0; i < sURLVariables.length; i++) {
		var sParameterName = sURLVariables[i].split('=');
		if (sParameterName[0] == sParam)
			return sParameterName[1];
	}
	return '';
} //end function rrGetQueryStringParam3

///////////////////////////////////////////////
//Structural function called by the admin_*config document.ready function.
//When the querystring has an "open" parameter, determine the image ID
//associated with that section.
function rrGetImageId3 (formSection) {
	switch (formSection) {
		case 'ped': return 'plus0';
		case 'desc': return 'plus1';
		case 'rel': return 'plus2';
		case 'time': return 'plus3';
		case 'peddesc': return 'plus4';
		case 'db': return 'plus0';
		case 'tables': return 'plus1';
		case 'folders': return 'plus2';
		case 'site': return 'plus3';
		case 'media': return 'plus4';
		case 'lang': return 'plus5';
		case 'priv': return 'plus6';
		case 'names': return 'plus7';
		case 'cemeteries': return 'plus8';
		case 'mailreg': return 'plus9';
		case 'pref': return 'plus10';
		case 'mobile': return 'plus11';
		case 'dna': return 'plus12';
		case 'misc': return 'plus13';
	} //end switch
} //end function rrGetImageId3

//////////////////////////////////////////////////
//function rrModOptionsOpenWikiPage opens a TNG Wiki page as a popup.
//It is invoked by the Wikipedia icon next the mod name that serves as a heading
//for that mod's options.
function rrModOptionsOpenWikiPage3(article, anchor='') {
	newwindow = window.open('https://tng.lythgoes.net/wiki/index.php?title='+article+'#'+anchor, 'wikiwindow', 'height=600, width=780, top=50, left=100, resizable=yes, scrollbars=yes, status=yes');
	newwindow.focus();
	return false;
} //end function rrModOptionsOpenWikiPage3
</script>

<style>
/************* Styles for Robin's Mod Options **************/
/* (Version number suffixes will need to be added to the classnames if their rules change!?!)
/* The dummy table that contains the options form field definitions before they
are moved into the programs form. */
.rrConfigTable {display:none;}
/* rrModWrap wraps each mod's content */
.rrModWrapper {box-shadow: 0 0 0 2px grey;
	border-radius:5px;
	padding:1px 3px;}
.rrModWrapper h4 {margin:0;}
.rrModWrapper tr {vertical-align:top;}
.rrModWrapper input[type='checkbox'] {position:relative; top:2px;}
.rrModWrapper input[type='radio'] {position:relative; top:2px;}
.rrModTable {clear:both;}
/* Right justify the option field prompts, but not mod names, which are the
first elements in each row of a subordinate table. */
.rrModTable>tbody>tr>td:first-child {text-align:right;
	padding-right:4px; padding-top:4px;}
/* Little field-button-like buttons to set option default values */
.rrSetupDefaultButton {padding:1px;
	font-size:95%;
	font-weight:bold;
	border-radius:3px;}
/* For the wiki logo in the block headings */
.rrModAdminWiki {cursor:pointer;}
</style>

